"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.capitalize = capitalize;
exports.getComposerName = getComposerName;
exports.default = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/slicedToArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/defineProperty"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/toConsumableArray"));

var _entries = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/entries"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/array/is-array"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/keys"));

var _node = require("graphql-compose-mongoose/node8");

function capitalize(name) {
  if (!name) {
    return '';
  }

  return name[0].toUpperCase() + name.slice(1);
}

function getComposerName(model, modelName) {
  var composer = model.composer;
  var name = modelName || model.name;

  if (composer) {
    return capitalize(composer);
  }

  var singular = (name[name.length - 1] === 's' ? name.slice(0, -1) : name).replace(/-([a-z])/g, function (s, v) {
    return v.toUpperCase();
  });
  return capitalize(singular);
}

var makeComposers = function makeComposers(_ref) {
  var schemaComposer = _ref.schemaComposer,
      Models = _ref.models,
      scopeAccessResolver = _ref.scopeAccessResolver,
      exclude = _ref.exclude,
      refMap = _ref.refMap;
  var composers = {};
  (0, _keys.default)(Models).map(function (name) {
    var model = Models[name];
    var composerName = getComposerName(model, name);
    var composer = (0, _node.composeWithMongoose)(model, {
      name: composerName
    });
    composer.removeField('password');
    composers[composerName] = composer;
    composers[composerName].model = model;
    return {
      model: model,
      composer: composer
    };
  }).forEach(function (_ref2) {
    var model = _ref2.model,
        composer = _ref2.composer;
    var fields = (0, _keys.default)(model.schema.tree).map(function (key) {
      return [key, model.schema.tree[key]];
    });
    var refFields = [];

    var findNestedRefFields = function findNestedRefFields(fieldsToTest, path) {
      // Find all nested ref
      fieldsToTest.filter(function (field) {
        // Test the field, or the first element of the field if it's an array
        var toCheck = (0, _isArray.default)(field[1]) ? field[1][0] : field[1];

        if (field[0] === 'id') {
          // If the field name is "id" then it's a Mongoose internal field
          return false;
        }

        return !( // If it's a function, then it's a standard Mongoose type
        toCheck instanceof Function // If a "schemaName" is present, then it's a standard field
        || !!toCheck.schemaName // If "type" is a function and "ref" is not a string, then it's a Mongoose complex field
        || typeof toCheck.ref !== 'string' && toCheck.type instanceof Function);
      }).forEach(function (field) {
        var array = (0, _isArray.default)(field[1]);
        var toCheck = array ? field[1][0] : field[1];

        if (typeof toCheck.ref === 'string') {
          // If the ref argument is a string, then we have a ref field
          refFields.push({
            field: field[0],
            ref: toCheck.ref,
            path: path,
            array: array
          });
        } else {
          // Else, we have a nested object or array of nested object, so check all fields again
          findNestedRefFields((0, _entries.default)(toCheck), [].concat((0, _toConsumableArray2.default)(path), [field[0]]));
        }
      });
    };

    findNestedRefFields(fields, []); // Add all ref resolvers

    refFields.forEach(function (field) {
      var refComposer = composer; // Get the correct composer for deeply nested fields

      if (field.path.length) {
        field.path.forEach(function (path) {
          refComposer = refComposer.getFieldTC(path);
        });
      } // Get the related composer by the ref


      var relatedComposerName = getComposerName(refMap[field.ref]); // Add the relation

      refComposer.addRelation(field.field, {
        resolver: function resolver() {
          return composers[relatedComposerName].getResolver(field.array ? 'findByIds' : 'findById');
        },
        prepareArgs: (0, _defineProperty2.default)({}, field.array ? '_ids' : '_id', function (source) {
          return source[field.field];
        })
      });
    });
  });
  var resolvers = {
    Query: {
      get: ['findOne', ''],
      list: ['findMany', 'List'],
      count: ['count', 'Count']
    },
    Mutation: {
      create: ['createOne', 'Create'],
      createMany: ['createMany', 'CreateMany'],
      update: ['updateById', 'Update'],
      delete: ['removeById', 'Delete']
    }
  };
  (0, _keys.default)(composers).filter(function (name) {
    return !exclude || exclude.indexOf(name) < 0;
  }).map(function (name) {
    return [name, composers[name]];
  }).forEach(function (_ref3) {
    var _ref4 = (0, _slicedToArray2.default)(_ref3, 2),
        name = _ref4[0],
        composer = _ref4[1];

    var lowercase = name[0].toLowerCase() + name.slice(1);

    var createResolver = function createResolver(type) {
      return function (resolver) {
        var _resolvers$type$resol = (0, _slicedToArray2.default)(resolvers[type][resolver], 2),
            method = _resolvers$type$resol[0],
            suffix = _resolvers$type$resol[1];

        composer[resolver] = scopeAccessResolver(composers)(composer.getResolver(method), name, resolver); // eslint-disable-line

        composer[resolver].unscoped = composer.getResolver(method); // eslint-disable-line

        composer[resolver].args = composer.getResolver(method).getArgs(); // eslint-disable-line

        composer[resolver].type = composer.getResolver(method); // eslint-disable-line

        schemaComposer[type].addFields((0, _defineProperty2.default)({}, lowercase + suffix, composer[resolver]));
      };
    };

    (0, _keys.default)(resolvers).forEach(function (type) {
      (0, _keys.default)(resolvers[type]).forEach(createResolver(type));
    });
  });
  return composers;
};

var _default = makeComposers;
exports.default = _default;