"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs2/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.Types = void 0;

var _promise = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/promise"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/createClass"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/objectSpread"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/defineProperty"));

var _regenerator = _interopRequireDefault(require("@babel/runtime-corejs2/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/asyncToGenerator"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/keys"));

var _cluster = _interopRequireDefault(require("cluster"));

var _mongoose = _interopRequireDefault(require("mongoose"));

var _mongooseElasticsearchXp = _interopRequireDefault(require("mongoose-elasticsearch-xp"));

var _graphqlCompose = require("graphql-compose");

var _logger = _interopRequireDefault(require("@foundationjs/logger"));

var _composer = _interopRequireWildcard(require("./composer"));

_mongoose.default.Promise = global.Promise;

function composerName(model) {
  return (0, _composer.getComposerName)(model);
}

function modelName(name) {
  return name.replace(/^([a-z])/, function (f, l) {
    return l.toUpperCase();
  });
}
/**
 * Return a list of pre-configured composers
 * @param {object} composers - Map of composers
 * @param {object} context - Request context
 * @returns {object} - Map of context-injected composers
 */


function scopeComposers(composers, context) {
  var scopedComposers = {};
  var queryResolvers = ['get', 'list', 'count'];
  var recordResolvers = ['create', 'update'];
  var recordsResolvers = ['createMany'];
  var idResolvers = ['delete'];
  (0, _keys.default)(composers).forEach(function (key) {
    var composer = composers[key];
    scopedComposers[key] = {
      model: composers[key].model
    };
    queryResolvers.forEach(function (rs) {
      scopedComposers[key][rs] = function (filter) {
        var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
            skip = _ref.skip,
            sort = _ref.sort,
            limit = _ref.limit;

        return composer[rs].resolve({
          args: {
            filter: filter,
            skip: skip,
            sort: sort,
            limit: limit
          },
          context: context
        });
      };

      scopedComposers[key][rs].unscoped = function (filter) {
        var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
            skip = _ref2.skip,
            sort = _ref2.sort,
            limit = _ref2.limit;

        return composer[rs].unscoped.resolve({
          args: {
            filter: filter,
            skip: skip,
            sort: sort,
            limit: limit
          },
          context: context
        });
      };
    });
    recordResolvers.forEach(function (rs) {
      scopedComposers[key][rs] = function (record) {
        return composer[rs].resolve({
          args: {
            record: record
          },
          context: context
        });
      };

      scopedComposers[key][rs].unscoped = function (record) {
        return composer[rs].unscoped.resolve({
          args: {
            record: record
          },
          context: context
        });
      };
    });
    recordsResolvers.forEach(function (rs) {
      scopedComposers[key][rs] = function (records) {
        return composer[rs].resolve({
          args: {
            records: records
          },
          context: context
        });
      };

      scopedComposers[key][rs].unscoped = function (records) {
        return composer[rs].unscoped.resolve({
          args: {
            records: records
          },
          context: context
        });
      };
    });
    idResolvers.forEach(function (rs) {
      scopedComposers[key][rs] = function (_id) {
        return composer[rs].resolve({
          args: {
            _id: _id
          },
          context: context
        });
      };

      scopedComposers[key][rs].unscoped = function (_id) {
        return composer[rs].unscoped.resolve({
          args: {
            _id: _id
          },
          context: context
        });
      };
    });
  });
  return scopedComposers;
}
/**
 * Add a field to a composer, with scoped request
 * @param {string} field - Name of the field to add
 * @param {string} kind - Whether it's a field, a query or a mutation
 * @param {*} to - Composer to add the field to
 * @param {*} composers - Map of composers to inject
 */


function addField(field, kind, to, composers) {
  var config = kind[field];
  to.addFields((0, _defineProperty2.default)({}, field, {
    projection: config.needs || config.projection,
    type: config.type || (config.extends ? config.extends.type : undefined),
    args: config.args || (config.extends ? config.extends.args : undefined),
    resolve: function () {
      var _resolve = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee(source, args, context, info) {
        return _regenerator.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                return _context.abrupt("return", config.resolve({
                  source: source,
                  args: args,
                  context: context,
                  info: info,
                  composers: scopeComposers(composers, context)
                }));

              case 1:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function resolve(_x, _x2, _x3, _x4) {
        return _resolve.apply(this, arguments);
      }

      return resolve;
    }()
  }));
}

var Types = (0, _objectSpread2.default)({}, _mongoose.default.Schema.Types, {
  Map: _mongoose.default.Schema.Types.Mixed
  /**
   * Class Persistence: initialize a Persistence configuration for a Foundation App.
   */

});
exports.Types = Types;

var Persistence =
/*#__PURE__*/
function () {
  /**
   * Constructor
   * @param {PersistenceConfiguration} config - Optional config object
   */
  function Persistence(config) {
    var _this = this;

    (0, _classCallCheck2.default)(this, Persistence);
    (0, _defineProperty2.default)(this, "logger", void 0);
    (0, _defineProperty2.default)(this, "schema", void 0);
    (0, _defineProperty2.default)(this, "io", void 0);
    (0, _defineProperty2.default)(this, "typeComposers", void 0);
    (0, _defineProperty2.default)(this, "Mutation", void 0);
    (0, _defineProperty2.default)(this, "Query", void 0);
    (0, _defineProperty2.default)(this, "configureIO", function (_ref3) {
      var io = _ref3.io;
      var logger = _this.logger;
      logger.info('New Socket.IO service available. Connecting...');
      _this.io = io;
      io.on('connection', function (socket) {
        socket.join('updates');
      });
      logger.info('Socket.IO connected.');
    });

    if (config) {
      this.init(config);
    }
  }
  /**
   * Function configuring the Persistence instance
   * @param {models} models - List of models to store
   * @param {query} query - Optional object to configure additional queries
   * @param {mutation} mutation - Optional object to configure additional mutations
   * @param {log} log - Optional object to configure the logger
   * @param {endpoint} endpoint - Optional object to configure the endpoint for Mongo
   * @returns {boolean} - Successful initialization
   */


  (0, _createClass2.default)(Persistence, [{
    key: "init",
    value: function () {
      var _init = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee3(_ref4) {
        var _this2 = this;

        var modelsConfig, query, mutation, log, endpoint, logConfig, logger, scopes, models, refMap, scopeAccessResolver, composers, typeComposers;
        return _regenerator.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                modelsConfig = _ref4.models, query = _ref4.query, mutation = _ref4.mutation, log = _ref4.log, endpoint = _ref4.endpoint;
                // Prepare the logger
                logConfig = log || {};

                if (!_cluster.default.isMaster) {
                  logConfig.filename = logConfig.filename && "[".concat(_cluster.default.worker.id, "]_").concat(logConfig.filename);
                } // Prepare the logger


                this.logger = new _logger.default('Persistence', logConfig);
                logger = this.logger;

                if (!_cluster.default.isMaster) {
                  logger.info("Cluster Mode! Instance ".concat(_cluster.default.worker.id, " running"));
                  logger.level = 'error';
                }

                logger.info('Initializing Persistence');
                scopes = {};
                models = {};
                refMap = {};

                if (modelsConfig.length) {
                  _context3.next = 13;
                  break;
                }

                logger.error('Cannot initialize Persistence without any model!');
                return _context3.abrupt("return", false);

              case 13:
                logger.info("Found ".concat(modelsConfig.length, " models. Importing..."));
                modelsConfig.forEach(function (model) {
                  scopes[composerName(model)] = model.scope;
                  refMap[model.name] = model;
                  var params;

                  if (model.elasticsearch && model.elasticsearch.fields) {
                    params = {
                      es_extend: model.elasticsearch.fields
                    };
                  }

                  var schema = new _mongoose.default.Schema(model.schema, params);
                  var name = modelName(model.name);
                  schema.post('save', function (doc) {
                    logger.debug("".concat(name, " updated"));

                    if (model.onPostSave) {
                      model.onPostSave(doc);
                    }

                    if (_this2.io) {
                      _this2.io.to('updates').emit("".concat(name.toLowerCase(), "-updated"), doc._id);

                      _this2.io.to('updates').emit("".concat(name.toLowerCase(), "-saved"), doc._id);
                    }
                  });
                  schema.post('remove', function (doc) {
                    logger.debug("".concat(name, " updated"));

                    if (model.onPostRemove) {
                      model.onPostRemove(doc);
                    }

                    if (_this2.io) {
                      _this2.io.to('updates').emit("".concat(name.toLowerCase(), "-updated"), doc._id);

                      _this2.io.to('updates').emit("".concat(name.toLowerCase(), "-removed"), doc._id);
                    }
                  });

                  if (model.elasticsearch) {
                    schema.plugin(_mongooseElasticsearchXp.default, {
                      hydrate: true
                    });
                  }

                  models[name] = _mongoose.default.model(model.name, schema, model.collection); // Force the collection name if provided

                  models[name].composer = model.composer;
                  logger.info("Model '".concat(name, "' imported."));
                });

                scopeAccessResolver = function scopeAccessResolver(composers) {
                  return function (resolver, name, type) {
                    return resolver.wrapResolve(function (next) {
                      return (
                        /*#__PURE__*/
                        function () {
                          var _ref6 = (0, _asyncToGenerator2.default)(
                          /*#__PURE__*/
                          _regenerator.default.mark(function _callee2(_ref5) {
                            var source, args, context, info, newArgs;
                            return _regenerator.default.wrap(function _callee2$(_context2) {
                              while (1) {
                                switch (_context2.prev = _context2.next) {
                                  case 0:
                                    source = _ref5.source, args = _ref5.args, context = _ref5.context, info = _ref5.info;

                                    if (!(!scopes[name] || !(scopes[name] instanceof Function))) {
                                      _context2.next = 3;
                                      break;
                                    }

                                    return _context2.abrupt("return", next({
                                      source: source,
                                      args: args,
                                      context: context,
                                      info: info
                                    }));

                                  case 3:
                                    _context2.next = 5;
                                    return scopes[name]({
                                      source: source,
                                      args: args,
                                      context: context,
                                      info: info,
                                      models: models,
                                      // Deprecated
                                      composers: scopeComposers(composers, context),
                                      type: type
                                    });

                                  case 5:
                                    _context2.t0 = _context2.sent;

                                    if (_context2.t0) {
                                      _context2.next = 8;
                                      break;
                                    }

                                    _context2.t0 = args;

                                  case 8:
                                    newArgs = _context2.t0;
                                    return _context2.abrupt("return", next({
                                      source: source,
                                      args: newArgs,
                                      context: context,
                                      info: info
                                    }));

                                  case 10:
                                  case "end":
                                    return _context2.stop();
                                }
                              }
                            }, _callee2, this);
                          }));

                          return function (_x6) {
                            return _ref6.apply(this, arguments);
                          };
                        }()
                      );
                    });
                  };
                };

                composers = (0, _composer.default)({
                  schemaComposer: _graphqlCompose.schemaComposer,
                  models: models,
                  scopeAccessResolver: scopeAccessResolver,
                  refMap: refMap
                });

                if (query) {
                  logger.info('Importing custom Query...');
                  query({
                    schemaComposer: _graphqlCompose.schemaComposer,
                    composers: composers,
                    scopeAccessResolver: scopeAccessResolver(composers),
                    models: models
                  });
                  logger.info('Imported.');
                }

                if (mutation) {
                  logger.info('Importing custom Mutation...');
                  mutation({
                    schemaComposer: _graphqlCompose.schemaComposer,
                    composers: composers,
                    scopeAccessResolver: scopeAccessResolver(composers),
                    models: models
                  });
                  logger.info('Imported.');
                }

                typeComposers = {};
                (0, _keys.default)(composers).forEach(function (comp) {
                  typeComposers[comp] = composers[comp]; // Deprecated

                  typeComposers["".concat(comp, "TC")] = composers[comp];
                });
                this.typeComposers = typeComposers;
                this.Query = _graphqlCompose.schemaComposer.Query;
                this.Mutation = _graphqlCompose.schemaComposer.Mutation;
                logger.info('Adding custom fields to models...');
                modelsConfig.forEach(function (model) {
                  var fields = model.fields;

                  if (fields) {
                    var fieldsConfig = fields({
                      schemaComposer: _graphqlCompose.schemaComposer,
                      // Deprecated
                      composers: composers,
                      // Deprecated
                      scopeAccessResolver: scopeAccessResolver(composers),
                      // Deprecated
                      models: models,
                      // Deprecated
                      typeComposers: typeComposers
                    });

                    if (fieldsConfig) {
                      if (fieldsConfig.fields) {
                        (0, _keys.default)(fieldsConfig.fields).forEach(function (field) {
                          return addField(field, fieldsConfig.fields, composers[composerName(model)], composers);
                        });
                      }

                      if (fieldsConfig.queries) {
                        (0, _keys.default)(fieldsConfig.queries).forEach(function (field) {
                          return addField(field, fieldsConfig.queries, _graphqlCompose.schemaComposer.Query, composers);
                        });
                      }

                      if (fieldsConfig.mutations) {
                        (0, _keys.default)(fieldsConfig.mutations).forEach(function (field) {
                          return addField(field, fieldsConfig.mutations, _graphqlCompose.schemaComposer.Mutation, composers);
                        });
                      }
                    }
                  }
                });
                logger.info('Done.');
                logger.info('Building GraphQL schema...');
                this.schema = _graphqlCompose.schemaComposer.buildSchema();
                logger.info('GraphQL schema available.');

                if (!endpoint) {
                  _context3.next = 32;
                  break;
                }

                return _context3.abrupt("return", this.connect({
                  endpoint: endpoint
                }));

              case 32:
                return _context3.abrupt("return", true);

              case 33:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function init(_x5) {
        return _init.apply(this, arguments);
      }

      return init;
    }()
  }, {
    key: "connect",
    value: function () {
      var _connect = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee4(_ref7) {
        var _this3 = this;

        var endpoint;
        return _regenerator.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                endpoint = _ref7.endpoint;
                return _context4.abrupt("return", new _promise.default(function (resolve, reject) {
                  var logger = _this3.logger;
                  logger.info("Connecting to Mongo at ".concat(endpoint, "."));

                  _mongoose.default.connect(endpoint, {
                    useNewUrlParser: true
                  }, function () {
                    logger.info('Mongo connected.');
                    resolve(true);
                  }, function (error) {
                    reject(error);
                  });
                }));

              case 2:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function connect(_x7) {
        return _connect.apply(this, arguments);
      }

      return connect;
    }()
  }]);
  return Persistence;
}();

exports.default = Persistence;